
//-----------------------------------------------------------------------------------------------------
// included header files:
//--------------------

#include "Watchdog.h"
#include <zephyr/logging/log.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/types.h>
#include <zephyr/pm/pm.h>  
#include <zephyr/pm/device.h>
#include <zephyr/pm/device_runtime.h>
#include <zephyr/drivers/watchdog.h>
#include <stdbool.h>
//-----------------------------------------------------------------------------------------------------
// global defines:
//--------------------

LOG_MODULE_REGISTER(WatchDog, 4);

//-----------------------------------------------------------------------------------------------------
// global type definitions:
//-----------------------
#define WATCHDOG_MIN_WINDOW 0U //<< 0 seconds
#define WATCHDOG_MAX_WINDOW 30000U //<< 30 seconds

//-----------------------------------------------------------------------------------------------------
// global variables:
//-------------------------

const struct device *const wdt = DEVICE_DT_GET(DT_ALIAS(watchdog0));
int wdt_channel_id;

//-----------------------------------------------------------------------------------------------------
// class and function Prototypes:
//-----


static void wdt_callback(const struct device *wdt_dev, int channel_id);

/*
 * 	@Description	: To initialize the watchdog timer
 * 	@param          : None
 * 	@Return	        : err
 */
int watchdog_init()
{
    
    if (!device_is_ready(wdt)) {
		LOG_ERR("%s: device not ready.\n", wdt->name);
		return -EIO;
	}

    struct wdt_timeout_cfg wdt_config = {
		/* Reset SoC when watchdog timer expires. */
		.flags = WDT_FLAG_RESET_SOC,

		/* Expire watchdog after max window */
		.window.min = WATCHDOG_MIN_WINDOW,
		.window.max = WATCHDOG_MAX_WINDOW,
	};

    wdt_config.callback = wdt_callback; //<< Not configuring call back

    wdt_channel_id = wdt_install_timeout(wdt, &wdt_config);
	    if (wdt_channel_id < 0) {
		LOG_ERR("Watchdog install error\n %d ", wdt_channel_id);
	}
	return wdt_channel_id;

}

/*
 * 	@Description	: To start the watchdog timer
 * 	@param          : None
 * 	@Return	        : err
 */
int watchdog_start()
{
    int err;
    
    err = wdt_setup(wdt, WDT_OPT_PAUSE_HALTED_BY_DBG); //<< watchdog started, but will be halted during debugging. 
	if (err < 0) {
		LOG_ERR("Watchdog setup error - %d\n", err);
	}
		return err;

}


/*
 * 	@Description	: To stop the watchdog timer
 * 	@param          : None
 * 	@Return	        : err
 */
int watchdog_stop()
{
    int err;
    
    err = wdt_disable(wdt);
	if (err < 0) {
		LOG_ERR("Watchdog stop resulted in error %d \n", err);
	}
		return err;

}


/*
 * 	@Description	: To feed the watchdog
 * 	@param          : None
 * 	@Return	        : err
 */
int watchdog_feed()
{
    int err;
    err = wdt_feed(wdt, wdt_channel_id);

    if(err<0)
        LOG_ERR("WDT Feed failed %d", err);

    return err;

}

static void wdt_callback(const struct device *wdt_dev, int channel_id)
{
	static bool handled_event;

	if (handled_event) {
		return;
	}

	wdt_feed(wdt_dev, channel_id);

	LOG_DBG("Handled things..ready to reset\n");

	handled_event = true;
}